VERSION 5.00
Begin {C62A69F0-16DC-11CE-9E98-00AA00574A4F} AnnotateSelectedFeatures 
   Caption         =   "Annotate Selected Features"
   ClientHeight    =   3015
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   4290
   OleObjectBlob   =   "AnnotateSelectedFeatures.frx":0000
   StartUpPosition =   1  'CenterOwner
End
Attribute VB_Name = "AnnotateSelectedFeatures"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Private m_pFeatureLayer As IFeatureLayer
Private m_pAnnoClasses As esriSystem.IArray

Private Sub UserForm_Activate()

   ' Check for selected feature layer

   Dim pDoc As IMxDocument
   Dim pSelItem As IUnknown

   Set m_pFeatureLayer = Nothing
   Set m_pAnnoClasses = Nothing
   lbAnnoClasses.Clear

   Set pDoc = ThisDocument
   Set pSelItem = pDoc.SelectedItem
   If pSelItem Is Nothing Then
      MsgBox "Nothing selected", vbExclamation
      Me.Hide
      Exit Sub
   ElseIf TypeOf pSelItem Is IFeatureLayer Then
      Set m_pFeatureLayer = pSelItem
   Else
      MsgBox "No feature layer selected", vbExclamation
      Me.Hide
      Exit Sub
   End If
   
   ' Check for selected features
   
   Dim pFSelection As IFeatureSelection
   
   Set pFSelection = m_pFeatureLayer
   If pFSelection.SelectionSet.Count = 0 Then
      MsgBox "No features selected", vbExclamation
      Me.Hide
      Exit Sub
   End If
   
   ' Check that feature class is being edited
   
   Dim pFClass As IFeatureClass
   Dim pDSEdit As IDatasetEdit
   
   Set pFClass = m_pFeatureLayer.FeatureClass
   Set pDSEdit = pFClass
   If Not pDSEdit.IsBeingEdited Then
      MsgBox "Target dataset is not being edited", vbExclamation
      Me.Hide
      Exit Sub
   End If
   
   ' Check for feature-linked annotation classes
   
   Dim pERC As IEnumRelationshipClass
   Dim pRC As IRelationshipClass
   Dim pDClass As IObjectClass
   Dim pFClass2 As IFeatureClass
          
   Set m_pAnnoClasses = New esriSystem.Array
   Set pERC = pFClass.RelationshipClasses(esriRelRoleAny)
   Set pRC = pERC.Next
   Do While Not pRC Is Nothing
      Set pDClass = pRC.DestinationClass
      Set pFClass2 = pDClass
      If TypeOf pFClass2.Extension Is IAnnoClass Then
         m_pAnnoClasses.Add pFClass2
      End If
      Set pRC = pERC.Next
   Loop
   If m_pAnnoClasses.Count = 0 Then
      MsgBox "No feature-linked annotation classes", vbExclamation
      Me.Hide
      Exit Sub
   End If
   
   ' Populate lbAnnoClasses
   
   Dim i As Long, j As Long, k As Long
   Dim pDataset As IDataset
   Dim pAnnoClass As IAnnoClass
   Dim pAPC As IAnnotateLayerPropertiesCollection
   Dim pAP As IAnnotateLayerProperties
   Dim sItem As String
   
   k = -1
   For i = 0 To m_pAnnoClasses.Count - 1
      Set pFClass2 = m_pAnnoClasses.Element(i)
      Set pDataset = pFClass2
      Set pAnnoClass = pFClass2.Extension
      Set pAPC = pAnnoClass.AnnoProperties
      iNumAP = pAPC.Count
      For j = 0 To iNumAP - 1
         sItem = pDataset.Name
         pAPC.QueryItem j, pAP
         sItem = sItem & " - " & pAP.Class
         lbAnnoClasses.AddItem sItem
         k = k + 1
         lbAnnoClasses.Selected(k) = True
      Next
   Next

End Sub

Private Sub btnOK_Click()

   ' Start edit operation
   
   Dim pEditor As IEditor
   Dim pID As New UID
   pID = "esriEditor.Editor"
   Set pEditor = Application.FindExtensionByCLSID(pID)
   pEditor.StartOperation

   ' Populate lbAnnoClasses
   
   Dim i As Long, j As Long, k As Long
   Dim iNumAnnos As Long, iResult As Long
   Dim pAnnoFClass As IFeatureClass
   Dim pAnnoClass As IAnnoClass
   Dim pAPC As IAnnotateLayerPropertiesCollection
   Dim pAP As IAnnotateLayerProperties
   
   iNumAnnos = 0
   k = -1
   For i = 0 To m_pAnnoClasses.Count - 1
      Set pAnnoFClass = m_pAnnoClasses.Element(i)
      Set pAnnoClass = pAnnoFClass.Extension
      Set pAPC = pAnnoClass.AnnoProperties
      iNumAP = pAPC.Count
      For j = 0 To iNumAP - 1
         pAPC.QueryItem j, pAP
         k = k + 1
         If lbAnnoClasses.Selected(k) Then
            iResult = AddAnnosForClass(pAnnoFClass, pAP, j)
            iNumAnnos = iNumAnnos + iResult
         End If
      Next
   Next
   
   pEditor.StopOperation "Annotate Selected Features"
   
   Dim pDoc As IMxDocument
   
   Set pDoc = ThisDocument
   pDoc.ActiveView.Refresh
   MsgBox "Annotations added: " & iNumAnnos
   Me.Hide

End Sub

Private Sub btnCancel_Click()

   Me.Hide

End Sub

Private Function AddAnnosForClass(pAnnoFClass As IFeatureClass, pAP As IAnnotateLayerProperties, iClassNum As Long) As Long
   
   ' Get annotation parameters

   Dim pDoc As IMxDocument
   Dim pMap As IMap
   Dim pMapOverposter As IMapOverposter
   Dim pBarrierCollection As IBarrierCollection
   Dim pAnnotateFeature As IAnnotateFeature2
   
   Set pDoc = ThisDocument
   Set pMap = pDoc.FocusMap
   Set pMapOverposter = pMap
   Set pBarrierCollection = pMap.Barriers(pDoc.ActiveView.extent)
   Set pAnnotateFeature = New AnnotateFeature
   
   ' Loop through selected features
   
   Dim pFSelection As IFeatureSelection
   Dim pSel As ISelectionSet
   Dim pQF As IQueryFilter
   Dim pCursor As ICursor
   Dim pRow As iRow
   Dim iNumAnnos As Long
   Dim pPlacedElement As IElement, pUnplacedElement As IElement
   Dim pFeat As iFeature
   Dim pAnnoFeat As IAnnotationFeature2
   
   Set pFSelection = m_pFeatureLayer
   Set pSel = pFSelection.SelectionSet
   Set pQF = Nothing
   pSel.Search pQF, False, pCursor
   Set pRow = pCursor.NextRow
   iNumAnnos = 0
   Do While Not pRow Is Nothing
      pAnnotateFeature.Label pMapOverposter.OverposterProperties, pAP, _
         pRow, pDoc.ActiveView.ScreenDisplay, pMap.SpatialReference, _
         pBarrierCollection, pPlacedElement, pUnplacedElement
      If Not pPlacedElement Is Nothing Then
         Set pFeat = pAnnoFClass.CreateFeature
         Set pAnnoFeat = pFeat
         pAnnoFeat.Annotation = pPlacedElement
         pAnnoFeat.LinkedFeatureID = pRow.OID
         pAnnoFeat.AnnotationClassID = iClassNum
         pAnnoFeat.Status = esriAnnoStatusPlaced
         pFeat.Store
         iNumAnnos = iNumAnnos + 1
      End If
      Set pRow = pCursor.NextRow
   Loop
   
   AddAnnosForClass = iNumAnnos

End Function
